# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from ..types import (
    Model,
    model_get_params,
    model_list_params,
    model_create_params,
    model_update_params,
    model_prepare_params,
    model_validate_upload_params,
    model_get_upload_endpoint_params,
    model_get_download_endpoint_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorModels, AsyncCursorModels
from ..types.model import Model
from .._base_client import AsyncPaginator, make_request_options
from ..types.model_param import ModelParam
from ..types.type_date_param import TypeDateParam
from ..types.peft_details_param import PeftDetailsParam
from ..types.base_model_details_param import BaseModelDetailsParam
from ..types.conversation_config_param import ConversationConfigParam
from ..types.model_get_upload_endpoint_response import ModelGetUploadEndpointResponse
from ..types.model_get_download_endpoint_response import ModelGetDownloadEndpointResponse

__all__ = ["ModelsResource", "AsyncModelsResource"]


class ModelsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return ModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return ModelsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str | None = None,
        model_id: str,
        cluster: str | Omit = omit,
        model: ModelParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Model:
        """
        Create Model

        Args:
          model_id: ID of the model.

          cluster: The resource name of the BYOC cluster to which this model belongs. e.g.
              accounts/my-account/clusters/my-cluster. Empty if it belongs to a Fireworks
              cluster.

          model: The properties of the Model being created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/models"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models",
            body=maybe_transform(
                {
                    "model_id": model_id,
                    "cluster": cluster,
                    "model": model,
                },
                model_create_params.ModelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Model,
        )

    def update(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        base_model_details: BaseModelDetailsParam | Omit = omit,
        context_length: int | Omit = omit,
        conversation_config: ConversationConfigParam | Omit = omit,
        default_draft_model: str | Omit = omit,
        default_draft_token_count: int | Omit = omit,
        deprecation_date: TypeDateParam | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        github_url: str | Omit = omit,
        hugging_face_url: str | Omit = omit,
        kind: Literal[
            "KIND_UNSPECIFIED",
            "HF_BASE_MODEL",
            "HF_PEFT_ADDON",
            "HF_TEFT_ADDON",
            "FLUMINA_BASE_MODEL",
            "FLUMINA_ADDON",
            "DRAFT_ADDON",
            "FIRE_AGENT",
            "LIVE_MERGE",
            "CUSTOM_MODEL",
            "EMBEDDING_MODEL",
            "SNAPSHOT_MODEL",
        ]
        | Omit = omit,
        peft_details: PeftDetailsParam | Omit = omit,
        public: bool | Omit = omit,
        snapshot_type: Literal["FULL_SNAPSHOT", "INCREMENTAL_SNAPSHOT"] | Omit = omit,
        supports_image_input: bool | Omit = omit,
        supports_lora: bool | Omit = omit,
        supports_tools: bool | Omit = omit,
        teft_details: object | Omit = omit,
        training_context_length: int | Omit = omit,
        use_hf_apply_chat_template: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Model:
        """Update Model

        Args:
          base_model_details: Base model details.

        Required if kind is HF_BASE_MODEL. Must not be set
              otherwise.

          context_length: The maximum context length supported by the model.

          conversation_config: If set, the Chat Completions API will be enabled for this model.

          default_draft_model: The default draft model to use when creating a deployment. If empty, speculative
              decoding is disabled by default.

          default_draft_token_count: The default draft token count to use when creating a deployment. Must be
              specified if default_draft_model is specified.

          deprecation_date: If specified, this is the date when the serverless deployment of the model will
              be taken down.

          description: The description of the model. Must be fewer than 1000 characters long.

          display_name: Human-readable display name of the model. e.g. "My Model" Must be fewer than 64
              characters long.

          github_url: The URL to GitHub repository of the model.

          hugging_face_url: The URL to the Hugging Face model.

          kind: The kind of model. If not specified, the default is HF_PEFT_ADDON.

          peft_details: PEFT addon details. Required if kind is HF_PEFT_ADDON or HF_TEFT_ADDON.

          public: If true, the model will be publicly readable.

          supports_image_input: If set, images can be provided as input to the model.

          supports_lora: Whether this model supports LoRA.

          supports_tools: If set, tools (i.e. functions) can be provided as input to the model, and the
              model may respond with one or more tool calls.

          teft_details: TEFT addon details. Required if kind is HF_TEFT_ADDON. Must not be set
              otherwise.

          training_context_length: The maximum context length supported by the model.

          use_hf_apply_chat_template: If true, the model will use the Hugging Face apply_chat_template API to apply
              the chat template.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._patch(
            f"/v1/accounts/{account_id}/models/{model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}",
            body=maybe_transform(
                {
                    "base_model_details": base_model_details,
                    "context_length": context_length,
                    "conversation_config": conversation_config,
                    "default_draft_model": default_draft_model,
                    "default_draft_token_count": default_draft_token_count,
                    "deprecation_date": deprecation_date,
                    "description": description,
                    "display_name": display_name,
                    "github_url": github_url,
                    "hugging_face_url": hugging_face_url,
                    "kind": kind,
                    "peft_details": peft_details,
                    "public": public,
                    "snapshot_type": snapshot_type,
                    "supports_image_input": supports_image_input,
                    "supports_lora": supports_lora,
                    "supports_tools": supports_tools,
                    "teft_details": teft_details,
                    "training_context_length": training_context_length,
                    "use_hf_apply_chat_template": use_hf_apply_chat_template,
                },
                model_update_params.ModelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Model,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorModels[Model]:
        """
        List Models

        Args:
          filter: Only model satisfying the provided filter (if specified) will be returned. See
              https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of models to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListModels call. Provide this to retrieve
              the subsequent page. When paginating, all other parameters provided to
              ListModels must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            f"/v1/accounts/{account_id}/models"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models",
            page=SyncCursorModels[Model],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    model_list_params.ModelListParams,
                ),
            ),
            model=Model,
        )

    def delete(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Model

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._delete(
            f"/v1/accounts/{account_id}/models/{model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def get(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Model:
        """Get Model

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/models/{model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"read_mask": read_mask}, model_get_params.ModelGetParams),
            ),
            cast_to=Model,
        )

    def get_download_endpoint(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelGetDownloadEndpointResponse:
        """
        Get Model Download Endpoint

        Args:
          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/models/{model_id}:getDownloadEndpoint"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:getDownloadEndpoint",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"read_mask": read_mask}, model_get_download_endpoint_params.ModelGetDownloadEndpointParams
                ),
            ),
            cast_to=ModelGetDownloadEndpointResponse,
        )

    def get_upload_endpoint(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        filename_to_size: Dict[str, str],
        enable_resumable_upload: bool | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelGetUploadEndpointResponse:
        """
        Get Model Upload Endpoint

        Args:
          filename_to_size: A mapping from the file name to its size in bytes.

          enable_resumable_upload: If true, enable resumable upload instead of PUT.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/models/{model_id}:getUploadEndpoint"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:getUploadEndpoint",
            body=maybe_transform(
                {
                    "filename_to_size": filename_to_size,
                    "enable_resumable_upload": enable_resumable_upload,
                    "read_mask": read_mask,
                },
                model_get_upload_endpoint_params.ModelGetUploadEndpointParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGetUploadEndpointResponse,
        )

    def prepare(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        precision: Literal[
            "PRECISION_UNSPECIFIED",
            "FP16",
            "FP8",
            "FP8_MM",
            "FP8_AR",
            "FP8_MM_KV_ATTN",
            "FP8_KV",
            "FP8_MM_V2",
            "FP8_V2",
            "FP8_MM_KV_ATTN_V2",
            "NF4",
            "FP4",
            "BF16",
            "FP4_BLOCKSCALED_MM",
            "FP4_MX_MOE",
        ]
        | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Prepare Model for different precisions

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/models/{model_id}:prepare"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:prepare",
            body=maybe_transform(
                {
                    "precision": precision,
                    "read_mask": read_mask,
                },
                model_prepare_params.ModelPrepareParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def validate_upload(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        config_only: bool | Omit = omit,
        skip_hf_config_validation: bool | Omit = omit,
        trust_remote_code: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Validate Model Upload

        Args:
          config_only: If true, skip tokenizer and parameter name validation.

          skip_hf_config_validation: If true, skip the Hugging Face config validation.

          trust_remote_code: If true, trusts remote code when validating the Hugging Face config.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/models/{model_id}:validateUpload"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:validateUpload",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "config_only": config_only,
                        "skip_hf_config_validation": skip_hf_config_validation,
                        "trust_remote_code": trust_remote_code,
                    },
                    model_validate_upload_params.ModelValidateUploadParams,
                ),
            ),
            cast_to=object,
        )


class AsyncModelsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncModelsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str | None = None,
        model_id: str,
        cluster: str | Omit = omit,
        model: ModelParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Model:
        """
        Create Model

        Args:
          model_id: ID of the model.

          cluster: The resource name of the BYOC cluster to which this model belongs. e.g.
              accounts/my-account/clusters/my-cluster. Empty if it belongs to a Fireworks
              cluster.

          model: The properties of the Model being created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/models"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models",
            body=await async_maybe_transform(
                {
                    "model_id": model_id,
                    "cluster": cluster,
                    "model": model,
                },
                model_create_params.ModelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Model,
        )

    async def update(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        base_model_details: BaseModelDetailsParam | Omit = omit,
        context_length: int | Omit = omit,
        conversation_config: ConversationConfigParam | Omit = omit,
        default_draft_model: str | Omit = omit,
        default_draft_token_count: int | Omit = omit,
        deprecation_date: TypeDateParam | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        github_url: str | Omit = omit,
        hugging_face_url: str | Omit = omit,
        kind: Literal[
            "KIND_UNSPECIFIED",
            "HF_BASE_MODEL",
            "HF_PEFT_ADDON",
            "HF_TEFT_ADDON",
            "FLUMINA_BASE_MODEL",
            "FLUMINA_ADDON",
            "DRAFT_ADDON",
            "FIRE_AGENT",
            "LIVE_MERGE",
            "CUSTOM_MODEL",
            "EMBEDDING_MODEL",
            "SNAPSHOT_MODEL",
        ]
        | Omit = omit,
        peft_details: PeftDetailsParam | Omit = omit,
        public: bool | Omit = omit,
        snapshot_type: Literal["FULL_SNAPSHOT", "INCREMENTAL_SNAPSHOT"] | Omit = omit,
        supports_image_input: bool | Omit = omit,
        supports_lora: bool | Omit = omit,
        supports_tools: bool | Omit = omit,
        teft_details: object | Omit = omit,
        training_context_length: int | Omit = omit,
        use_hf_apply_chat_template: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Model:
        """Update Model

        Args:
          base_model_details: Base model details.

        Required if kind is HF_BASE_MODEL. Must not be set
              otherwise.

          context_length: The maximum context length supported by the model.

          conversation_config: If set, the Chat Completions API will be enabled for this model.

          default_draft_model: The default draft model to use when creating a deployment. If empty, speculative
              decoding is disabled by default.

          default_draft_token_count: The default draft token count to use when creating a deployment. Must be
              specified if default_draft_model is specified.

          deprecation_date: If specified, this is the date when the serverless deployment of the model will
              be taken down.

          description: The description of the model. Must be fewer than 1000 characters long.

          display_name: Human-readable display name of the model. e.g. "My Model" Must be fewer than 64
              characters long.

          github_url: The URL to GitHub repository of the model.

          hugging_face_url: The URL to the Hugging Face model.

          kind: The kind of model. If not specified, the default is HF_PEFT_ADDON.

          peft_details: PEFT addon details. Required if kind is HF_PEFT_ADDON or HF_TEFT_ADDON.

          public: If true, the model will be publicly readable.

          supports_image_input: If set, images can be provided as input to the model.

          supports_lora: Whether this model supports LoRA.

          supports_tools: If set, tools (i.e. functions) can be provided as input to the model, and the
              model may respond with one or more tool calls.

          teft_details: TEFT addon details. Required if kind is HF_TEFT_ADDON. Must not be set
              otherwise.

          training_context_length: The maximum context length supported by the model.

          use_hf_apply_chat_template: If true, the model will use the Hugging Face apply_chat_template API to apply
              the chat template.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._patch(
            f"/v1/accounts/{account_id}/models/{model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}",
            body=await async_maybe_transform(
                {
                    "base_model_details": base_model_details,
                    "context_length": context_length,
                    "conversation_config": conversation_config,
                    "default_draft_model": default_draft_model,
                    "default_draft_token_count": default_draft_token_count,
                    "deprecation_date": deprecation_date,
                    "description": description,
                    "display_name": display_name,
                    "github_url": github_url,
                    "hugging_face_url": hugging_face_url,
                    "kind": kind,
                    "peft_details": peft_details,
                    "public": public,
                    "snapshot_type": snapshot_type,
                    "supports_image_input": supports_image_input,
                    "supports_lora": supports_lora,
                    "supports_tools": supports_tools,
                    "teft_details": teft_details,
                    "training_context_length": training_context_length,
                    "use_hf_apply_chat_template": use_hf_apply_chat_template,
                },
                model_update_params.ModelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Model,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Model, AsyncCursorModels[Model]]:
        """
        List Models

        Args:
          filter: Only model satisfying the provided filter (if specified) will be returned. See
              https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of models to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListModels call. Provide this to retrieve
              the subsequent page. When paginating, all other parameters provided to
              ListModels must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            f"/v1/accounts/{account_id}/models"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models",
            page=AsyncCursorModels[Model],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    model_list_params.ModelListParams,
                ),
            ),
            model=Model,
        )

    async def delete(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Model

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._delete(
            f"/v1/accounts/{account_id}/models/{model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def get(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Model:
        """Get Model

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/models/{model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"read_mask": read_mask}, model_get_params.ModelGetParams),
            ),
            cast_to=Model,
        )

    async def get_download_endpoint(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelGetDownloadEndpointResponse:
        """
        Get Model Download Endpoint

        Args:
          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/models/{model_id}:getDownloadEndpoint"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:getDownloadEndpoint",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"read_mask": read_mask}, model_get_download_endpoint_params.ModelGetDownloadEndpointParams
                ),
            ),
            cast_to=ModelGetDownloadEndpointResponse,
        )

    async def get_upload_endpoint(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        filename_to_size: Dict[str, str],
        enable_resumable_upload: bool | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelGetUploadEndpointResponse:
        """
        Get Model Upload Endpoint

        Args:
          filename_to_size: A mapping from the file name to its size in bytes.

          enable_resumable_upload: If true, enable resumable upload instead of PUT.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/models/{model_id}:getUploadEndpoint"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:getUploadEndpoint",
            body=await async_maybe_transform(
                {
                    "filename_to_size": filename_to_size,
                    "enable_resumable_upload": enable_resumable_upload,
                    "read_mask": read_mask,
                },
                model_get_upload_endpoint_params.ModelGetUploadEndpointParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGetUploadEndpointResponse,
        )

    async def prepare(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        precision: Literal[
            "PRECISION_UNSPECIFIED",
            "FP16",
            "FP8",
            "FP8_MM",
            "FP8_AR",
            "FP8_MM_KV_ATTN",
            "FP8_KV",
            "FP8_MM_V2",
            "FP8_V2",
            "FP8_MM_KV_ATTN_V2",
            "NF4",
            "FP4",
            "BF16",
            "FP4_BLOCKSCALED_MM",
            "FP4_MX_MOE",
        ]
        | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Prepare Model for different precisions

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/models/{model_id}:prepare"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:prepare",
            body=await async_maybe_transform(
                {
                    "precision": precision,
                    "read_mask": read_mask,
                },
                model_prepare_params.ModelPrepareParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def validate_upload(
        self,
        model_id: str,
        *,
        account_id: str | None = None,
        config_only: bool | Omit = omit,
        skip_hf_config_validation: bool | Omit = omit,
        trust_remote_code: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Validate Model Upload

        Args:
          config_only: If true, skip tokenizer and parameter name validation.

          skip_hf_config_validation: If true, skip the Hugging Face config validation.

          trust_remote_code: If true, trusts remote code when validating the Hugging Face config.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/models/{model_id}:validateUpload"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/models/{model_id}:validateUpload",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "config_only": config_only,
                        "skip_hf_config_validation": skip_hf_config_validation,
                        "trust_remote_code": trust_remote_code,
                    },
                    model_validate_upload_params.ModelValidateUploadParams,
                ),
            ),
            cast_to=object,
        )


class ModelsResourceWithRawResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.create = to_raw_response_wrapper(
            models.create,
        )
        self.update = to_raw_response_wrapper(
            models.update,
        )
        self.list = to_raw_response_wrapper(
            models.list,
        )
        self.delete = to_raw_response_wrapper(
            models.delete,
        )
        self.get = to_raw_response_wrapper(
            models.get,
        )
        self.get_download_endpoint = to_raw_response_wrapper(
            models.get_download_endpoint,
        )
        self.get_upload_endpoint = to_raw_response_wrapper(
            models.get_upload_endpoint,
        )
        self.prepare = to_raw_response_wrapper(
            models.prepare,
        )
        self.validate_upload = to_raw_response_wrapper(
            models.validate_upload,
        )


class AsyncModelsResourceWithRawResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.create = async_to_raw_response_wrapper(
            models.create,
        )
        self.update = async_to_raw_response_wrapper(
            models.update,
        )
        self.list = async_to_raw_response_wrapper(
            models.list,
        )
        self.delete = async_to_raw_response_wrapper(
            models.delete,
        )
        self.get = async_to_raw_response_wrapper(
            models.get,
        )
        self.get_download_endpoint = async_to_raw_response_wrapper(
            models.get_download_endpoint,
        )
        self.get_upload_endpoint = async_to_raw_response_wrapper(
            models.get_upload_endpoint,
        )
        self.prepare = async_to_raw_response_wrapper(
            models.prepare,
        )
        self.validate_upload = async_to_raw_response_wrapper(
            models.validate_upload,
        )


class ModelsResourceWithStreamingResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.create = to_streamed_response_wrapper(
            models.create,
        )
        self.update = to_streamed_response_wrapper(
            models.update,
        )
        self.list = to_streamed_response_wrapper(
            models.list,
        )
        self.delete = to_streamed_response_wrapper(
            models.delete,
        )
        self.get = to_streamed_response_wrapper(
            models.get,
        )
        self.get_download_endpoint = to_streamed_response_wrapper(
            models.get_download_endpoint,
        )
        self.get_upload_endpoint = to_streamed_response_wrapper(
            models.get_upload_endpoint,
        )
        self.prepare = to_streamed_response_wrapper(
            models.prepare,
        )
        self.validate_upload = to_streamed_response_wrapper(
            models.validate_upload,
        )


class AsyncModelsResourceWithStreamingResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.create = async_to_streamed_response_wrapper(
            models.create,
        )
        self.update = async_to_streamed_response_wrapper(
            models.update,
        )
        self.list = async_to_streamed_response_wrapper(
            models.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            models.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            models.get,
        )
        self.get_download_endpoint = async_to_streamed_response_wrapper(
            models.get_download_endpoint,
        )
        self.get_upload_endpoint = async_to_streamed_response_wrapper(
            models.get_upload_endpoint,
        )
        self.prepare = async_to_streamed_response_wrapper(
            models.prepare,
        )
        self.validate_upload = async_to_streamed_response_wrapper(
            models.validate_upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["TypeDate"]


class TypeDate(BaseModel):
    """
    * A full date, with non-zero year, month, and day values
    * A month and day value, with a zero year, such as an anniversary
    * A year on its own, with zero month and day values
    * A year and month value, with a zero day, such as a credit card expiration
    date

    Related types are [google.type.TimeOfDay][google.type.TimeOfDay] and
    `google.protobuf.Timestamp`.
    """

    day: Optional[int] = None
    """Day of a month.

    Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by
    itself or a year and month where the day isn't significant.
    """

    month: Optional[int] = None
    """Month of a year.

    Must be from 1 to 12, or 0 to specify a year without a month and day.
    """

    year: Optional[int] = None
    """Year of the date.

    Must be from 1 to 9999, or 0 to specify a date without a year.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

from pydantic import Field as FieldInfo

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = [
    "SyncCursorAccounts",
    "AsyncCursorAccounts",
    "SyncCursorAPIKeys",
    "AsyncCursorAPIKeys",
    "SyncCursorBatchInferenceJobs",
    "AsyncCursorBatchInferenceJobs",
    "SyncCursorDatasets",
    "AsyncCursorDatasets",
    "SyncCursorDeploymentShapeVersions",
    "AsyncCursorDeploymentShapeVersions",
    "SyncCursorDeployments",
    "AsyncCursorDeployments",
    "SyncCursorDpoJobs",
    "AsyncCursorDpoJobs",
    "SyncCursorEvaluationJobs",
    "AsyncCursorEvaluationJobs",
    "SyncCursorEvaluators",
    "AsyncCursorEvaluators",
    "SyncCursorLora",
    "AsyncCursorLora",
    "SyncCursorModels",
    "AsyncCursorModels",
    "SyncCursorReinforcementFineTuningJobs",
    "AsyncCursorReinforcementFineTuningJobs",
    "SyncCursorReinforcementFineTuningSteps",
    "AsyncCursorReinforcementFineTuningSteps",
    "SyncCursorSecrets",
    "AsyncCursorSecrets",
    "SyncCursorSupervisedFineTuningJobs",
    "AsyncCursorSupervisedFineTuningJobs",
    "SyncCursorUsers",
    "AsyncCursorUsers",
]

_T = TypeVar("_T")


class SyncCursorAccounts(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    accounts: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        accounts = self.accounts
        if not accounts:
            return []
        return accounts

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorAccounts(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    accounts: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        accounts = self.accounts
        if not accounts:
            return []
        return accounts

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorAPIKeys(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    api_keys: List[_T] = FieldInfo(alias="apiKeys")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        api_keys = self.api_keys
        if not api_keys:
            return []
        return api_keys

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorAPIKeys(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    api_keys: List[_T] = FieldInfo(alias="apiKeys")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        api_keys = self.api_keys
        if not api_keys:
            return []
        return api_keys

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorBatchInferenceJobs(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    batch_inference_jobs: List[_T] = FieldInfo(alias="batchInferenceJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        batch_inference_jobs = self.batch_inference_jobs
        if not batch_inference_jobs:
            return []
        return batch_inference_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorBatchInferenceJobs(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    batch_inference_jobs: List[_T] = FieldInfo(alias="batchInferenceJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        batch_inference_jobs = self.batch_inference_jobs
        if not batch_inference_jobs:
            return []
        return batch_inference_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorDatasets(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    datasets: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        datasets = self.datasets
        if not datasets:
            return []
        return datasets

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorDatasets(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    datasets: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        datasets = self.datasets
        if not datasets:
            return []
        return datasets

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorDeploymentShapeVersions(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    deployment_shape_versions: List[_T] = FieldInfo(alias="deploymentShapeVersions")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        deployment_shape_versions = self.deployment_shape_versions
        if not deployment_shape_versions:
            return []
        return deployment_shape_versions

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorDeploymentShapeVersions(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    deployment_shape_versions: List[_T] = FieldInfo(alias="deploymentShapeVersions")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        deployment_shape_versions = self.deployment_shape_versions
        if not deployment_shape_versions:
            return []
        return deployment_shape_versions

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorDeployments(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    deployments: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        deployments = self.deployments
        if not deployments:
            return []
        return deployments

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorDeployments(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    deployments: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        deployments = self.deployments
        if not deployments:
            return []
        return deployments

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorDpoJobs(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    dpo_jobs: List[_T] = FieldInfo(alias="dpoJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        dpo_jobs = self.dpo_jobs
        if not dpo_jobs:
            return []
        return dpo_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorDpoJobs(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    dpo_jobs: List[_T] = FieldInfo(alias="dpoJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        dpo_jobs = self.dpo_jobs
        if not dpo_jobs:
            return []
        return dpo_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorEvaluationJobs(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    evaluation_jobs: List[_T] = FieldInfo(alias="evaluationJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        evaluation_jobs = self.evaluation_jobs
        if not evaluation_jobs:
            return []
        return evaluation_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorEvaluationJobs(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    evaluation_jobs: List[_T] = FieldInfo(alias="evaluationJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        evaluation_jobs = self.evaluation_jobs
        if not evaluation_jobs:
            return []
        return evaluation_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorEvaluators(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    evaluators: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        evaluators = self.evaluators
        if not evaluators:
            return []
        return evaluators

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorEvaluators(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    evaluators: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        evaluators = self.evaluators
        if not evaluators:
            return []
        return evaluators

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorLora(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    deployed_models: List[_T] = FieldInfo(alias="deployedModels")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        deployed_models = self.deployed_models
        if not deployed_models:
            return []
        return deployed_models

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorLora(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    deployed_models: List[_T] = FieldInfo(alias="deployedModels")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        deployed_models = self.deployed_models
        if not deployed_models:
            return []
        return deployed_models

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorModels(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    models: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        models = self.models
        if not models:
            return []
        return models

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorModels(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    models: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        models = self.models
        if not models:
            return []
        return models

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorReinforcementFineTuningJobs(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    reinforcement_fine_tuning_jobs: List[_T] = FieldInfo(alias="reinforcementFineTuningJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        reinforcement_fine_tuning_jobs = self.reinforcement_fine_tuning_jobs
        if not reinforcement_fine_tuning_jobs:
            return []
        return reinforcement_fine_tuning_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorReinforcementFineTuningJobs(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    reinforcement_fine_tuning_jobs: List[_T] = FieldInfo(alias="reinforcementFineTuningJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        reinforcement_fine_tuning_jobs = self.reinforcement_fine_tuning_jobs
        if not reinforcement_fine_tuning_jobs:
            return []
        return reinforcement_fine_tuning_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorReinforcementFineTuningSteps(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    rlor_trainer_jobs: List[_T] = FieldInfo(alias="rlorTrainerJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        rlor_trainer_jobs = self.rlor_trainer_jobs
        if not rlor_trainer_jobs:
            return []
        return rlor_trainer_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorReinforcementFineTuningSteps(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    rlor_trainer_jobs: List[_T] = FieldInfo(alias="rlorTrainerJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        rlor_trainer_jobs = self.rlor_trainer_jobs
        if not rlor_trainer_jobs:
            return []
        return rlor_trainer_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorSecrets(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    secrets: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        secrets = self.secrets
        if not secrets:
            return []
        return secrets

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorSecrets(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    secrets: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        secrets = self.secrets
        if not secrets:
            return []
        return secrets

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorSupervisedFineTuningJobs(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    supervised_fine_tuning_jobs: List[_T] = FieldInfo(alias="supervisedFineTuningJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        supervised_fine_tuning_jobs = self.supervised_fine_tuning_jobs
        if not supervised_fine_tuning_jobs:
            return []
        return supervised_fine_tuning_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorSupervisedFineTuningJobs(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    supervised_fine_tuning_jobs: List[_T] = FieldInfo(alias="supervisedFineTuningJobs")
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        supervised_fine_tuning_jobs = self.supervised_fine_tuning_jobs
        if not supervised_fine_tuning_jobs:
            return []
        return supervised_fine_tuning_jobs

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class SyncCursorUsers(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    users: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        users = self.users
        if not users:
            return []
        return users

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})


class AsyncCursorUsers(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    users: List[_T]
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        users = self.users
        if not users:
            return []
        return users

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page_token = self.next_page_token
        if not next_page_token:
            return None

        return PageInfo(params={"pageToken": next_page_token})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from fireworks import Fireworks, AsyncFireworks
from tests.utils import assert_matches_type
from fireworks.types import (
    Model,
    ModelGetUploadEndpointResponse,
    ModelGetDownloadEndpointResponse,
)
from fireworks.pagination import SyncCursorModels, AsyncCursorModels

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestModels:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Fireworks) -> None:
        model = client.models.create(
            account_id="account_id",
            model_id="modelId",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Fireworks) -> None:
        model = client.models.create(
            account_id="account_id",
            model_id="modelId",
            cluster="cluster",
            model={
                "base_model_details": {
                    "checkpoint_format": "CHECKPOINT_FORMAT_UNSPECIFIED",
                    "model_type": "modelType",
                    "moe": True,
                    "parameter_count": "parameterCount",
                    "supports_fireattention": True,
                    "supports_mtp": True,
                    "tunable": True,
                    "world_size": 0,
                },
                "context_length": 0,
                "conversation_config": {
                    "style": "style",
                    "system": "system",
                    "template": "template",
                },
                "default_draft_model": "defaultDraftModel",
                "default_draft_token_count": 0,
                "deprecation_date": {
                    "day": 0,
                    "month": 0,
                    "year": 0,
                },
                "description": "description",
                "display_name": "displayName",
                "github_url": "githubUrl",
                "hugging_face_url": "huggingFaceUrl",
                "kind": "KIND_UNSPECIFIED",
                "peft_details": {
                    "base_model": "baseModel",
                    "r": 0,
                    "target_modules": ["string"],
                    "merge_addon_model_name": "mergeAddonModelName",
                },
                "public": True,
                "snapshot_type": "FULL_SNAPSHOT",
                "supports_image_input": True,
                "supports_lora": True,
                "supports_tools": True,
                "teft_details": {},
                "training_context_length": 0,
                "use_hf_apply_chat_template": True,
            },
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.create(
            account_id="account_id",
            model_id="modelId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.create(
            account_id="account_id",
            model_id="modelId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(Model, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.create(
                account_id="",
                model_id="modelId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Fireworks) -> None:
        model = client.models.update(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Fireworks) -> None:
        model = client.models.update(
            model_id="model_id",
            account_id="account_id",
            base_model_details={
                "checkpoint_format": "CHECKPOINT_FORMAT_UNSPECIFIED",
                "model_type": "modelType",
                "moe": True,
                "parameter_count": "parameterCount",
                "supports_fireattention": True,
                "supports_mtp": True,
                "tunable": True,
                "world_size": 0,
            },
            context_length=0,
            conversation_config={
                "style": "style",
                "system": "system",
                "template": "template",
            },
            default_draft_model="defaultDraftModel",
            default_draft_token_count=0,
            deprecation_date={
                "day": 0,
                "month": 0,
                "year": 0,
            },
            description="description",
            display_name="displayName",
            github_url="githubUrl",
            hugging_face_url="huggingFaceUrl",
            kind="KIND_UNSPECIFIED",
            peft_details={
                "base_model": "baseModel",
                "r": 0,
                "target_modules": ["string"],
                "merge_addon_model_name": "mergeAddonModelName",
            },
            public=True,
            snapshot_type="FULL_SNAPSHOT",
            supports_image_input=True,
            supports_lora=True,
            supports_tools=True,
            teft_details={},
            training_context_length=0,
            use_hf_apply_chat_template=True,
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.update(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.update(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(Model, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.update(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.update(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Fireworks) -> None:
        model = client.models.list(
            account_id="account_id",
        )
        assert_matches_type(SyncCursorModels[Model], model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Fireworks) -> None:
        model = client.models.list(
            account_id="account_id",
            filter="filter",
            order_by="orderBy",
            page_size=0,
            page_token="pageToken",
            read_mask="readMask",
        )
        assert_matches_type(SyncCursorModels[Model], model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.list(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(SyncCursorModels[Model], model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.list(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(SyncCursorModels[Model], model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.list(
                account_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Fireworks) -> None:
        model = client.models.delete(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.delete(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.delete(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(object, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.delete(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.delete(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get(self, client: Fireworks) -> None:
        model = client.models.get(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_with_all_params(self, client: Fireworks) -> None:
        model = client.models.get(
            model_id="model_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.get(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.get(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(Model, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.get(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.get(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_download_endpoint(self, client: Fireworks) -> None:
        model = client.models.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_download_endpoint_with_all_params(self, client: Fireworks) -> None:
        model = client.models.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_download_endpoint(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_download_endpoint(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_download_endpoint(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.get_download_endpoint(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.get_download_endpoint(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_upload_endpoint(self, client: Fireworks) -> None:
        model = client.models.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )
        assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_upload_endpoint_with_all_params(self, client: Fireworks) -> None:
        model = client.models.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
            enable_resumable_upload=True,
            read_mask="readMask",
        )
        assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_upload_endpoint(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_upload_endpoint(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_upload_endpoint(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.get_upload_endpoint(
                model_id="model_id",
                account_id="",
                filename_to_size={"foo": "string"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.get_upload_endpoint(
                model_id="",
                account_id="account_id",
                filename_to_size={"foo": "string"},
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_prepare(self, client: Fireworks) -> None:
        model = client.models.prepare(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_prepare_with_all_params(self, client: Fireworks) -> None:
        model = client.models.prepare(
            model_id="model_id",
            account_id="account_id",
            precision="PRECISION_UNSPECIFIED",
            read_mask="readMask",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_prepare(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.prepare(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_prepare(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.prepare(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(object, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_prepare(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.prepare(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.prepare(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_validate_upload(self, client: Fireworks) -> None:
        model = client.models.validate_upload(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_validate_upload_with_all_params(self, client: Fireworks) -> None:
        model = client.models.validate_upload(
            model_id="model_id",
            account_id="account_id",
            config_only=True,
            skip_hf_config_validation=True,
            trust_remote_code=True,
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_validate_upload(self, client: Fireworks) -> None:
        response = client.models.with_raw_response.validate_upload(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_validate_upload(self, client: Fireworks) -> None:
        with client.models.with_streaming_response.validate_upload(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(object, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_validate_upload(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.models.with_raw_response.validate_upload(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.validate_upload(
                model_id="",
                account_id="account_id",
            )


class TestAsyncModels:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.create(
            account_id="account_id",
            model_id="modelId",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.create(
            account_id="account_id",
            model_id="modelId",
            cluster="cluster",
            model={
                "base_model_details": {
                    "checkpoint_format": "CHECKPOINT_FORMAT_UNSPECIFIED",
                    "model_type": "modelType",
                    "moe": True,
                    "parameter_count": "parameterCount",
                    "supports_fireattention": True,
                    "supports_mtp": True,
                    "tunable": True,
                    "world_size": 0,
                },
                "context_length": 0,
                "conversation_config": {
                    "style": "style",
                    "system": "system",
                    "template": "template",
                },
                "default_draft_model": "defaultDraftModel",
                "default_draft_token_count": 0,
                "deprecation_date": {
                    "day": 0,
                    "month": 0,
                    "year": 0,
                },
                "description": "description",
                "display_name": "displayName",
                "github_url": "githubUrl",
                "hugging_face_url": "huggingFaceUrl",
                "kind": "KIND_UNSPECIFIED",
                "peft_details": {
                    "base_model": "baseModel",
                    "r": 0,
                    "target_modules": ["string"],
                    "merge_addon_model_name": "mergeAddonModelName",
                },
                "public": True,
                "snapshot_type": "FULL_SNAPSHOT",
                "supports_image_input": True,
                "supports_lora": True,
                "supports_tools": True,
                "teft_details": {},
                "training_context_length": 0,
                "use_hf_apply_chat_template": True,
            },
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.create(
            account_id="account_id",
            model_id="modelId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.create(
            account_id="account_id",
            model_id="modelId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(Model, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.create(
                account_id="",
                model_id="modelId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            account_id="account_id",
            base_model_details={
                "checkpoint_format": "CHECKPOINT_FORMAT_UNSPECIFIED",
                "model_type": "modelType",
                "moe": True,
                "parameter_count": "parameterCount",
                "supports_fireattention": True,
                "supports_mtp": True,
                "tunable": True,
                "world_size": 0,
            },
            context_length=0,
            conversation_config={
                "style": "style",
                "system": "system",
                "template": "template",
            },
            default_draft_model="defaultDraftModel",
            default_draft_token_count=0,
            deprecation_date={
                "day": 0,
                "month": 0,
                "year": 0,
            },
            description="description",
            display_name="displayName",
            github_url="githubUrl",
            hugging_face_url="huggingFaceUrl",
            kind="KIND_UNSPECIFIED",
            peft_details={
                "base_model": "baseModel",
                "r": 0,
                "target_modules": ["string"],
                "merge_addon_model_name": "mergeAddonModelName",
            },
            public=True,
            snapshot_type="FULL_SNAPSHOT",
            supports_image_input=True,
            supports_lora=True,
            supports_tools=True,
            teft_details={},
            training_context_length=0,
            use_hf_apply_chat_template=True,
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.update(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.update(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(Model, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.update(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.update(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.list(
            account_id="account_id",
        )
        assert_matches_type(AsyncCursorModels[Model], model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.list(
            account_id="account_id",
            filter="filter",
            order_by="orderBy",
            page_size=0,
            page_token="pageToken",
            read_mask="readMask",
        )
        assert_matches_type(AsyncCursorModels[Model], model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.list(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(AsyncCursorModels[Model], model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.list(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(AsyncCursorModels[Model], model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.list(
                account_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.delete(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.delete(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.delete(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(object, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.delete(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.delete(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.get(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.get(
            model_id="model_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.get(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(Model, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.get(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(Model, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.get(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.get(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_download_endpoint(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_download_endpoint_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_download_endpoint(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_download_endpoint(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.get_download_endpoint(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(ModelGetDownloadEndpointResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_download_endpoint(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.get_download_endpoint(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.get_download_endpoint(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )
        assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_upload_endpoint_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
            enable_resumable_upload=True,
            read_mask="readMask",
        )
        assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.get_upload_endpoint(
            model_id="model_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(ModelGetUploadEndpointResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.get_upload_endpoint(
                model_id="model_id",
                account_id="",
                filename_to_size={"foo": "string"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.get_upload_endpoint(
                model_id="",
                account_id="account_id",
                filename_to_size={"foo": "string"},
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_prepare(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.prepare(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_prepare_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.prepare(
            model_id="model_id",
            account_id="account_id",
            precision="PRECISION_UNSPECIFIED",
            read_mask="readMask",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_prepare(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.prepare(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_prepare(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.prepare(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(object, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_prepare(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.prepare(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.prepare(
                model_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_validate_upload(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.validate_upload(
            model_id="model_id",
            account_id="account_id",
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_validate_upload_with_all_params(self, async_client: AsyncFireworks) -> None:
        model = await async_client.models.validate_upload(
            model_id="model_id",
            account_id="account_id",
            config_only=True,
            skip_hf_config_validation=True,
            trust_remote_code=True,
        )
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_validate_upload(self, async_client: AsyncFireworks) -> None:
        response = await async_client.models.with_raw_response.validate_upload(
            model_id="model_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(object, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_validate_upload(self, async_client: AsyncFireworks) -> None:
        async with async_client.models.with_streaming_response.validate_upload(
            model_id="model_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(object, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_validate_upload(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.models.with_raw_response.validate_upload(
                model_id="model_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.validate_upload(
                model_id="",
                account_id="account_id",
            )



"""
Iterative Reinforcement Learning Workflow using the Fireworks SDK.

This script demonstrates an iterative reinforcement learning workflow where:
1. A deployment is created with hot reload enabled
2. For each RL step:
   - Generate rollouts using concurrent inference
   - Create and upload a dataset with rollouts and rewards
   - Run a reinforcement fine-tuning step
   - Wait for training completion
   - Hot reload the new LoRA adapter onto the deployment
   - Clean up the dataset
"""

from __future__ import annotations

import os
import json
import time
import random
import asyncio
import logging
from typing import Any
from collections import defaultdict

from dotenv import load_dotenv  # type: ignore[import-not-found]

from fireworks import AsyncFireworks
from fireworks._compat import model_dump

load_dotenv()

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s %(message)s",
    datefmt="%H:%M:%S",
)
logger = logging.getLogger(__name__)

# Suppress HTTP request logs from httpx
logging.getLogger("httpx").setLevel(logging.WARNING)


# Configuration
ACCOUNT_ID = os.environ["FIREWORKS_ACCOUNT_ID"]
BASE_MODEL = "accounts/fireworks/models/qwen2p5-7b-instruct"
DEPLOYMENT_ID = "my-base-deployment"
NUM_STEPS = 2
NUM_PROMPTS = 2
NUM_GENERATIONS_PER_PROMPT = 2
CONCURRENCY = 4
ROLLOUTS_DIR = "rollouts"

# Generate a unique run ID for this workflow execution
RUN_ID = int(time.time())


async def wait_for_deployment_ready(
    client: AsyncFireworks,
    deployment_id: str,
    timeout_seconds: int = 600,
    poll_interval: int = 10,
) -> None:
    """Wait for a deployment to be ready."""
    logger.info(f"Waiting for deployment {deployment_id} to be ready (timeout: {timeout_seconds}s)...")
    start_time = time.time()
    while time.time() - start_time < timeout_seconds:
        deployment = await client.deployments.get(
            deployment_id=deployment_id,
        )
        state = deployment.state
        elapsed = int(time.time() - start_time)
        logger.info(f"Deployment state: {state} (elapsed: {elapsed}s)")

        if state == "READY":
            logger.info("Deployment is ready!")
            return
        elif state in ("FAILED", "DELETED", "DELETING"):
            raise Exception(f"Deployment entered bad state: {state}")

        await asyncio.sleep(poll_interval)

    raise TimeoutError(f"Deployment did not become ready within {timeout_seconds} seconds")


async def create_or_get_deployment(
    client: AsyncFireworks,
    deployment_id: str,
    base_model: str,
) -> dict[str, Any]:
    """Create a deployment with hot reload enabled, or get existing one."""
    logger.info(f"Checking for existing deployment: {deployment_id}")
    try:
        deployment = await client.deployments.get(deployment_id=deployment_id)
        logger.info(f"Found existing deployment: {deployment.name}")
        return model_dump(deployment)
    except Exception:
        logger.info(f"Creating deployment {deployment_id} with hot reload enabled...")
        logger.info(f"  Base model: {base_model}")
        deployment = await client.deployments.create(
            base_model=base_model,
            deployment_id=deployment_id,
            enable_hot_reload_latest_addon=True,
            min_replica_count=1,
            max_replica_count=1,
            accelerator_type="NVIDIA_H100_80GB",
        )
        logger.info(f"Created deployment: {deployment.name}")
        return model_dump(deployment)


async def generate_rollouts_and_rewards(
    client: AsyncFireworks,
    model: str,
    num_prompts: int = 10,
    num_generations_per_prompt: int = 8,
    concurrency: int = 100,
) -> list[dict[str, Any]]:
    """
    Generate rollouts and compute rewards for the given model using concurrent generation.
    Each sample contains multiple generations for Policy Optimization.
    """
    semaphore = asyncio.Semaphore(concurrency)

    async def generate_single_response(prompt_id: int, generation_id: int) -> dict[str, Any]:
        """Generate a single response for a given prompt."""
        async with semaphore:
            from fireworks.types.shared_params.chat_message import ChatMessage

            messages: list[ChatMessage] = [{"role": "user", "content": f"What is {prompt_id} + {prompt_id}?"}]

            response = await client.chat.completions.create(
                model=model,
                messages=messages,
                temperature=1.5,  # Higher temperature for more diverse responses
            )

            assistant_message = response.choices[0].message.content or ""

            # Randomize the reward score
            reward = random.random()

            return {
                "prompt_id": prompt_id,
                "generation_id": generation_id,
                "messages": messages + [{"role": "assistant", "content": assistant_message}],
                "evals": {"score": reward},
            }

    # Create all generation tasks concurrently
    tasks: list[asyncio.Task[dict[str, Any]]] = []
    for prompt_id in range(num_prompts):
        for generation_id in range(num_generations_per_prompt):
            task = asyncio.create_task(generate_single_response(prompt_id, generation_id))
            tasks.append(task)

    # Execute all generations concurrently
    logger.info(f"Starting {len(tasks)} concurrent generations...")
    start_time = time.time()
    num_completed = 0
    results: list[dict[str, Any]] = []

    for coro in asyncio.as_completed(tasks):
        result = await coro
        results.append(result)
        num_completed += 1
        if num_completed % 10 == 0:
            elapsed = time.time() - start_time
            rate = num_completed / elapsed if elapsed > 0 else 0
            logger.info(f"Completed {num_completed}/{len(tasks)} generations ({rate:.1f}/s)")

    total_time = time.time() - start_time
    logger.info(f"All generations completed in {total_time:.1f}s")

    # Group results by prompt_id to create dataset rows
    prompt_generations_map: defaultdict[int, list[dict[str, Any]]] = defaultdict(list)
    for result in results:
        prompt_generations_map[result["prompt_id"]].append(result)

    dataset_rows: list[dict[str, Any]] = []
    for prompt_id in range(num_prompts):
        prompt_generations: list[dict[str, Any]] = prompt_generations_map[prompt_id]
        sample_generations: list[dict[str, Any]] = [
            {"messages": gen["messages"], "evals": gen["evals"]} for gen in prompt_generations
        ]
        dataset_rows.append({"samples": sample_generations})

    return dataset_rows


def save_rollouts_to_file(
    dataset_rows: list[dict[str, Any]],
    step: int,
) -> str:
    """Save rollouts to a local file for inspection."""
    os.makedirs(ROLLOUTS_DIR, exist_ok=True)
    filename = f"step-{step + 1}-rollouts-{int(time.time())}.jsonl"
    filepath = os.path.join(ROLLOUTS_DIR, filename)

    with open(filepath, "w") as f:
        for row in dataset_rows:
            f.write(json.dumps(row, indent=None) + "\n")

    file_size = os.path.getsize(filepath)
    logger.info(f"Saved rollouts to {filepath} ({file_size} bytes)")
    return filepath


def example_count(rollouts_filepath: str) -> int:
    """Count the number of examples (non-empty lines) in the rollouts file."""
    with open(rollouts_filepath) as f:
        return sum(1 for line in f if line.strip())


async def create_and_upload_dataset(
    client: AsyncFireworks,
    dataset_id: str,
    rollouts_filepath: str,
    timeout_seconds: int = 300,
    poll_interval: int = 2,
) -> str:
    """Create a dataset, upload from the saved rollouts file, and wait for it to be ready."""
    # Create the dataset
    logger.info(f"Creating dataset {dataset_id}...")
    dataset = await client.datasets.create(
        dataset_id=dataset_id,
        dataset={
            "display_name": f"RL Training Dataset - {dataset_id}",
            "example_count": str(example_count(rollouts_filepath)),
        },
    )
    logger.info(f"Created dataset: {dataset.name}")

    # Upload the rollouts file
    logger.info(f"Uploading dataset from {rollouts_filepath}...")
    with open(rollouts_filepath, "rb") as f:
        await client.datasets.upload(
            dataset_id=dataset_id,
            file=f,
        )
    logger.info("Dataset file uploaded, waiting for processing...")

    # Poll until dataset is ready
    start_time = time.time()
    while time.time() - start_time < timeout_seconds:
        dataset = await client.datasets.get(
            dataset_id=dataset_id,
        )
        state = dataset.state
        elapsed = int(time.time() - start_time)
        logger.info(f"Dataset state: {state} (elapsed: {elapsed}s)")

        if state == "READY":
            logger.info("Dataset is ready!")
            if dataset.name is None:
                raise ValueError("Dataset name is None")
            return dataset.name
        elif state in ("UPLOADING", "STATE_UNSPECIFIED"):
            await asyncio.sleep(poll_interval)
        else:
            raise Exception(f"Unexpected dataset state: {state}")

    raise TimeoutError(f"Dataset did not become ready within {timeout_seconds} seconds")


async def wait_for_training_completion(
    client: AsyncFireworks,
    job_id: str,
    timeout_seconds: int = 3600,
    poll_interval: int = 10,
) -> dict[str, Any]:
    """Wait for a reinforcement fine-tuning step to complete."""
    logger.info(f"Waiting for training job {job_id} to complete (timeout: {timeout_seconds}s)...")
    start_time = time.time()

    # Terminal failure states
    failure_states = {
        "JOB_STATE_FAILED",
        "JOB_STATE_FAILED_CLEANING_UP",
        "JOB_STATE_CANCELLED",
        "JOB_STATE_EXPIRED",
        "JOB_STATE_EXPIRED_CLEANING_UP",
        "JOB_STATE_DELETING_CLEANING_UP",
    }

    while time.time() - start_time < timeout_seconds:
        job = await client.reinforcement_fine_tuning_steps.get(
            rlor_trainer_job_id=job_id,
        )

        state = job.state
        elapsed = int(time.time() - start_time)
        logger.info(f"Training state: {state} (elapsed: {elapsed}s)")

        if state == "JOB_STATE_COMPLETED":
            total_time = time.time() - start_time
            logger.info(f"Training completed in {total_time:.1f}s!")
            return model_dump(job)
        elif state in failure_states:
            raise Exception(f"Training job entered bad state: {state}")

        await asyncio.sleep(poll_interval)

    raise TimeoutError(f"Training did not complete within {timeout_seconds} seconds")


async def wait_for_model_ready(
    client: AsyncFireworks,
    model_id: str,
    timeout_seconds: int = 600,
    poll_interval: int = 10,
) -> None:
    """Wait for a model to be ready after training."""
    logger.info(f"Waiting for model {model_id} to be ready (timeout: {timeout_seconds}s)...")
    start_time = time.time()

    while time.time() - start_time < timeout_seconds:
        model = await client.models.get(
            model_id=model_id,
        )

        state = model.state
        elapsed = int(time.time() - start_time)
        logger.info(f"Model state: {state} (elapsed: {elapsed}s)")

        if state == "READY":
            logger.info("Model is ready!")
            return
        # STATE_UNSPECIFIED and UPLOADING are transient states, continue waiting
        await asyncio.sleep(poll_interval)

    raise TimeoutError(f"Model did not become ready within {timeout_seconds} seconds")


async def load_lora_adapter(
    client: AsyncFireworks,
    deployment_name: str,
    model_name: str,
) -> None:
    """
    Load a LoRA adapter onto a deployment using hot reload.

    The replace_merged_addon=True flag merges the new addon to the base model
    while unmerging/deleting any existing addon in the deployment.
    """
    logger.info("Loading LoRA adapter onto deployment...")
    logger.info(f"  Model: {model_name}")
    logger.info(f"  Deployment: {deployment_name}")
    await client.lora.load(
        model=model_name,
        deployment=deployment_name,
        replace_merged_addon=True,
    )
    logger.info("LoRA adapter loaded successfully")


async def run_reinforcement_learning() -> None:
    """Main function to run the iterative reinforcement learning workflow."""
    # Use default production client
    client = AsyncFireworks()

    logger.info("=" * 60)
    logger.info("Iterative Reinforcement Learning Workflow")
    logger.info("=" * 60)

    # Step 1: Create or get the base deployment
    logger.info("[Step 0] Setting up base deployment...")
    await create_or_get_deployment(
        client=client,
        deployment_id=DEPLOYMENT_ID,
        base_model=BASE_MODEL,
    )

    # Wait for deployment to be ready
    await wait_for_deployment_ready(
        client=client,
        deployment_id=DEPLOYMENT_ID,
    )

    # Iterative reinforcement learning loop
    deployment_name = f"accounts/{ACCOUNT_ID}/deployments/{DEPLOYMENT_ID}"

    for step in range(NUM_STEPS):
        logger.info("=" * 60)
        logger.info(f"Starting reinforcement learning step {step + 1}/{NUM_STEPS}")
        logger.info("=" * 60)

        # Generate rollouts and rewards using the deployment
        # (after step 0, the deployment will have the LoRA adapter loaded via hot reload)
        logger.info(f"[Step {step + 1}.1] Generating rollouts and computing rewards...")
        dataset_rows = await generate_rollouts_and_rewards(
            client=client,
            model=deployment_name,
            num_prompts=NUM_PROMPTS,
            num_generations_per_prompt=NUM_GENERATIONS_PER_PROMPT,
            concurrency=CONCURRENCY,
        )

        # Save rollouts to local file for inspection
        rollouts_filepath = save_rollouts_to_file(dataset_rows, step)

        # Create and upload dataset
        dataset_id = f"rl-dataset-{RUN_ID}-step-{step + 1}"
        logger.info(f"[Step {step + 1}.2] Creating and uploading dataset...")
        dataset_name = await create_and_upload_dataset(
            client=client,
            dataset_id=dataset_id,
            rollouts_filepath=rollouts_filepath,
        )

        # Create reinforcement fine-tuning step
        output_model_name = f"accounts/{ACCOUNT_ID}/models/rl-model-{RUN_ID}-v{step + 1}"
        from fireworks.types.shared_params.training_config import TrainingConfig

        training_config: TrainingConfig = {
            "output_model": output_model_name,
            "epochs": 1,
            "learning_rate": 1e-5,
        }
        if step == 0:
            training_config["base_model"] = BASE_MODEL
        else:
            training_config["warm_start_from"] = f"accounts/{ACCOUNT_ID}/models/rl-model-{RUN_ID}-v{step}"

        job_id = f"rl-job-{RUN_ID}-step-{step + 1}"
        logger.info(f"[Step {step + 1}.3] Starting reinforcement fine-tuning step...")
        logger.info(f"Creating job with ID: {job_id}")
        job = await client.reinforcement_fine_tuning_steps.create(
            rlor_trainer_job_id=job_id,
            dataset=dataset_name,
            display_name=f"RL Step {step + 1}",
            training_config=training_config,
        )
        logger.info(f"Created training job: {job.name}")

        # Wait for training completion
        logger.info(f"[Step {step + 1}.4] Waiting for training to complete...")
        await wait_for_training_completion(client=client, job_id=job_id)

        # Wait for the output model to be ready
        output_model_id = f"rl-model-{RUN_ID}-v{step + 1}"
        logger.info(f"[Step {step + 1}.5] Waiting for model to be ready...")
        await wait_for_model_ready(client=client, model_id=output_model_id)

        # Hot reload the new LoRA adapter onto the deployment
        # This swaps the model on the deployment without restarting it
        logger.info(f"[Step {step + 1}.6] Hot reloading new LoRA adapter...")
        await load_lora_adapter(
            client=client,
            deployment_name=deployment_name,
            model_name=output_model_name,
        )

        logger.info(f"Step {step + 1} completed! Model {output_model_name} is now active.")

        # Step 8: Clean up dataset
        logger.info(f"[Step {step + 1}.7] Cleaning up dataset...")
        try:
            await client.datasets.delete(
                dataset_id=dataset_id,
            )
            logger.info(f"Deleted dataset: {dataset_id}")
        except Exception as e:
            logger.warning(f"Failed to delete dataset: {e}")

    logger.info("=" * 60)
    logger.info("Reinforcement learning complete!")
    logger.info("=" * 60)
    final_model_name = f"accounts/{ACCOUNT_ID}/models/rl-model-{RUN_ID}-v{NUM_STEPS}"
    logger.info(f"Final model: {final_model_name}")


if __name__ == "__main__":
    asyncio.run(run_reinforcement_learning())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.inference_parameters import InferenceParameters

__all__ = ["BatchInferenceJobCreateParams"]


class BatchInferenceJobCreateParams(TypedDict, total=False):
    account_id: str

    batch_inference_job_id: Annotated[str, PropertyInfo(alias="batchInferenceJobId")]
    """ID of the batch inference job."""

    continued_from_job_name: Annotated[str, PropertyInfo(alias="continuedFromJobName")]
    """
    The resource name of the batch inference job that this job continues from. Used
    for lineage tracking to understand job continuation chains.
    """

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    inference_parameters: Annotated[InferenceParameters, PropertyInfo(alias="inferenceParameters")]
    """Parameters controlling the inference process."""

    input_dataset_id: Annotated[str, PropertyInfo(alias="inputDatasetId")]
    """The name of the dataset used for inference.

    This is required, except when continued_from_job_name is specified.
    """

    model: str
    """The name of the model to use for inference.

    This is required, except when continued_from_job_name is specified.
    """

    output_dataset_id: Annotated[str, PropertyInfo(alias="outputDatasetId")]
    """The name of the dataset used for storing the results.

    This will also contain the error file.
    """

    precision: Literal[
        "PRECISION_UNSPECIFIED",
        "FP16",
        "FP8",
        "FP8_MM",
        "FP8_AR",
        "FP8_MM_KV_ATTN",
        "FP8_KV",
        "FP8_MM_V2",
        "FP8_V2",
        "FP8_MM_KV_ATTN_V2",
        "NF4",
        "FP4",
        "BF16",
        "FP4_BLOCKSCALED_MM",
        "FP4_MX_MOE",
    ]
    """
    The precision with which the model should be served. If PRECISION_UNSPECIFIED, a
    default will be chosen based on the model.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["EvaluationJobCreateParams", "EvaluationJob"]


class EvaluationJobCreateParams(TypedDict, total=False):
    account_id: str

    evaluation_job: Required[Annotated[EvaluationJob, PropertyInfo(alias="evaluationJob")]]

    evaluation_job_id: Annotated[str, PropertyInfo(alias="evaluationJobId")]

    leaderboard_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="leaderboardIds")]
    """Optional leaderboards to attach this job to upon creation."""


class EvaluationJob(TypedDict, total=False):
    evaluator: Required[str]
    """The fully-qualified resource name of the Evaluation used by this job.

    Format: accounts/{account_id}/evaluators/{evaluator_id}
    """

    input_dataset: Required[Annotated[str, PropertyInfo(alias="inputDataset")]]
    """The fully-qualified resource name of the input Dataset used by this job.

    Format: accounts/{account_id}/datasets/{dataset_id}
    """

    output_dataset: Required[Annotated[str, PropertyInfo(alias="outputDataset")]]
    """The fully-qualified resource name of the output Dataset created by this job.

    Format: accounts/{account_id}/datasets/{output_dataset_id}
    """

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    output_stats: Annotated[str, PropertyInfo(alias="outputStats")]
    """The output dataset's aggregated stats for the evaluation job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["InferenceParameters"]


class InferenceParameters(TypedDict, total=False):
    """Parameters for the inference requests."""

    extra_body: Annotated[str, PropertyInfo(alias="extraBody")]
    """
    Additional parameters for the inference request as a JSON string. For example:
    "{\"stop\": [\"\\n\"]}".
    """

    max_tokens: Annotated[int, PropertyInfo(alias="maxTokens")]
    """Maximum number of tokens to generate per response."""

    n: int
    """Number of response candidates to generate per input."""

    temperature: float
    """Sampling temperature, typically between 0 and 2."""

    top_k: Annotated[int, PropertyInfo(alias="topK")]
    """Top-k sampling parameter, limits the token selection to the top k tokens."""

    top_p: Annotated[float, PropertyInfo(alias="topP")]
    """Top-p sampling parameter, typically between 0 and 1."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["TrainingConfig"]


class TrainingConfig(TypedDict, total=False):
    base_model: Annotated[str, PropertyInfo(alias="baseModel")]
    """
    The name of the base model to be fine-tuned Only one of 'base_model' or
    'warm_start_from' should be specified.
    """

    batch_size: Annotated[int, PropertyInfo(alias="batchSize")]
    """The maximum packed number of tokens per batch for training in sequence packing."""

    epochs: int
    """The number of epochs to train for."""

    gradient_accumulation_steps: Annotated[int, PropertyInfo(alias="gradientAccumulationSteps")]

    jinja_template: Annotated[str, PropertyInfo(alias="jinjaTemplate")]

    learning_rate: Annotated[float, PropertyInfo(alias="learningRate")]
    """The learning rate used for training."""

    learning_rate_warmup_steps: Annotated[int, PropertyInfo(alias="learningRateWarmupSteps")]

    lora_rank: Annotated[int, PropertyInfo(alias="loraRank")]
    """The rank of the LoRA layers."""

    max_context_length: Annotated[int, PropertyInfo(alias="maxContextLength")]
    """The maximum context length to use with the model."""

    output_model: Annotated[str, PropertyInfo(alias="outputModel")]
    """The model ID to be assigned to the resulting fine-tuned model.

    If not specified, the job ID will be used.
    """

    region: Literal[
        "REGION_UNSPECIFIED",
        "US_IOWA_1",
        "US_VIRGINIA_1",
        "US_ILLINOIS_1",
        "AP_TOKYO_1",
        "US_ARIZONA_1",
        "US_TEXAS_1",
        "US_ILLINOIS_2",
        "EU_FRANKFURT_1",
        "US_TEXAS_2",
        "EU_ICELAND_1",
        "EU_ICELAND_2",
        "US_WASHINGTON_1",
        "US_WASHINGTON_2",
        "US_WASHINGTON_3",
        "AP_TOKYO_2",
        "US_CALIFORNIA_1",
        "US_UTAH_1",
        "US_GEORGIA_1",
        "US_GEORGIA_2",
        "US_WASHINGTON_4",
        "US_GEORGIA_3",
    ]
    """The region where the fine-tuning job is located."""

    warm_start_from: Annotated[str, PropertyInfo(alias="warmStartFrom")]
    """
    The PEFT addon model in Fireworks format to be fine-tuned from Only one of
    'base_model' or 'warm_start_from' should be specified.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import Dict, List, Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel
from ..shared.chat_message import ChatMessage

__all__ = [
    "CompletionCreateResponse",
    "Choice",
    "ChoiceLogprobs",
    "ChoiceLogprobsLogProbs",
    "ChoiceLogprobsNewLogProbs",
    "ChoiceLogprobsNewLogProbsContent",
    "ChoiceLogprobsNewLogProbsContentTopLogprob",
    "ChoiceRawOutput",
    "ChoiceRawOutputCompletionLogprobs",
    "ChoiceRawOutputCompletionLogprobsContent",
    "ChoiceRawOutputCompletionLogprobsContentTopLogprob",
    "Usage",
    "UsagePromptTokensDetails",
]


class ChoiceLogprobsLogProbs(BaseModel):
    """Legacy log probabilities format"""

    text_offset: Optional[List[int]] = None

    token_ids: Optional[List[int]] = None

    token_logprobs: Optional[List[float]] = None

    tokens: Optional[List[str]] = None

    top_logprobs: Optional[List[Dict[str, float]]] = None


class ChoiceLogprobsNewLogProbsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    token_id: int

    bytes: Optional[List[int]] = None


class ChoiceLogprobsNewLogProbsContent(BaseModel):
    token: str

    bytes: List[int]

    logprob: float

    sampling_logprob: Optional[float] = None

    text_offset: int

    token_id: int

    extra_logprobs: Optional[List[float]] = None

    extra_tokens: Optional[List[int]] = None

    last_activation: Optional[str] = None

    routing_matrix: Optional[str] = None

    top_logprobs: Optional[List[ChoiceLogprobsNewLogProbsContentTopLogprob]] = None


class ChoiceLogprobsNewLogProbs(BaseModel):
    """OpenAI-compatible log probabilities format"""

    content: Optional[List[ChoiceLogprobsNewLogProbsContent]] = None


ChoiceLogprobs: TypeAlias = Union[ChoiceLogprobsLogProbs, ChoiceLogprobsNewLogProbs, None]


class ChoiceRawOutputCompletionLogprobsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    token_id: int

    bytes: Optional[List[int]] = None


class ChoiceRawOutputCompletionLogprobsContent(BaseModel):
    token: str

    bytes: List[int]

    logprob: float

    sampling_logprob: Optional[float] = None

    text_offset: int

    token_id: int

    extra_logprobs: Optional[List[float]] = None

    extra_tokens: Optional[List[int]] = None

    last_activation: Optional[str] = None

    routing_matrix: Optional[str] = None

    top_logprobs: Optional[List[ChoiceRawOutputCompletionLogprobsContentTopLogprob]] = None


class ChoiceRawOutputCompletionLogprobs(BaseModel):
    """OpenAI-compatible log probabilities format"""

    content: Optional[List[ChoiceRawOutputCompletionLogprobsContent]] = None


class ChoiceRawOutput(BaseModel):
    """
    Extension of OpenAI that returns low-level interaction of what the model
    sees, including the formatted prompt and function calls
    """

    completion: str
    """Raw completion produced by the model before any tool calls are parsed"""

    prompt_fragments: List[Union[str, int]]
    """
    Pieces of the prompt (like individual messages) before truncation and
    concatenation. Depending on prompt_truncate_len some of the messages might be
    dropped. Contains a mix of strings to be tokenized and individual tokens (if
    dictated by the conversation template)
    """

    prompt_token_ids: List[int]
    """Fully processed prompt as seen by the model"""

    completion_logprobs: Optional[ChoiceRawOutputCompletionLogprobs] = None
    """OpenAI-compatible log probabilities format"""

    completion_token_ids: Optional[List[int]] = None
    """Token IDs for the raw completion"""

    grammar: Optional[str] = None
    """
    Grammar used for constrained decoding, can be either user provided (directly or
    JSON schema) or inferred by the chat template
    """

    images: Optional[List[str]] = None
    """Images in the prompt"""


class Choice(BaseModel):
    """A chat completion choice generated by a chat model.

    Attributes:
      index (int): The index of the chat completion choice.
      message (ChatMessage): The chat completion message.
      finish_reason (Optional[str]): The reason the model stopped generating tokens. This will be "stop" if
        the model hit a natural stop point or a provided stop sequence, or
        "length" if the maximum number of tokens specified in the request was
        reached.
      token_ids (Optional[List[int]]): Token IDs for the generated message content (when return_token_ids=true)
    """

    index: int

    message: ChatMessage
    """A chat completion message."""

    finish_reason: Optional[str] = None

    logprobs: Optional[ChoiceLogprobs] = None
    """Legacy log probabilities format"""

    raw_output: Optional[ChoiceRawOutput] = None
    """
    Extension of OpenAI that returns low-level interaction of what the model sees,
    including the formatted prompt and function calls
    """

    token_ids: Optional[List[int]] = None


class UsagePromptTokensDetails(BaseModel):
    """Details about prompt tokens, including cached tokens"""

    cached_tokens: Optional[int] = None


class Usage(BaseModel):
    """Usage statistics."""

    prompt_tokens: int
    """The number of tokens in the prompt"""

    total_tokens: int
    """The total number of tokens used in the request (prompt + completion)"""

    completion_tokens: Optional[int] = None
    """The number of tokens in the generated completion"""

    prompt_tokens_details: Optional[UsagePromptTokensDetails] = None
    """Details about prompt tokens, including cached tokens"""


class CompletionCreateResponse(BaseModel):
    """The response message from a /v1/chat/completions call."""

    id: str
    """A unique identifier of the response"""

    choices: List[Choice]
    """The list of chat completion choices"""

    created: int
    """The Unix time in seconds when the response was generated"""

    model: str
    """The model used for the chat completion"""

    object: Optional[str] = None
    """The object type, which is always "chat.completion" """

    perf_metrics: Optional[Dict[str, builtins.object]] = None
    """See parameter [perf_metrics_in_response](#body-perf-metrics-in-response)"""

    prompt_token_ids: Optional[List[int]] = None
    """Token IDs for the prompt (when return_token_ids=true)"""

    usage: Optional[Usage] = None
    """Usage statistics."""
